package org.gcube.portlets.widgets.exporter.client;




import org.gcube.portlets.d4sreporting.common.shared.Model;
import org.gcube.portlets.widgets.lighttree.client.ItemType;
import org.gcube.portlets.widgets.lighttree.client.event.PopupEvent;
import org.gcube.portlets.widgets.lighttree.client.event.PopupHandler;
import org.gcube.portlets.widgets.lighttree.client.save.WorkspaceLightTreeSavePopup;
import org.gcube.portlets.widgets.exporter.client.event.ExportingCompletedEvent;
import org.gcube.portlets.widgets.exporter.client.event.ReportExporterEvent;
import org.gcube.portlets.widgets.exporter.client.event.ReportExporterEvent.OperationResult;
import org.gcube.portlets.widgets.exporter.shared.SaveReportFileExistException;
import org.gcube.portlets.widgets.exporter.shared.TypeExporter;

import com.extjs.gxt.ui.client.event.ButtonEvent;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.event.MessageBoxEvent;
import com.extjs.gxt.ui.client.event.SelectionListener;
import com.extjs.gxt.ui.client.widget.Dialog;
import com.extjs.gxt.ui.client.widget.MessageBox;
import com.extjs.gxt.ui.client.widget.Window;
import com.extjs.gxt.ui.client.widget.button.Button;
import com.extjs.gxt.ui.client.widget.form.CheckBox;
import com.google.gwt.core.client.GWT;
import com.google.gwt.core.client.RunAsyncCallback;
import com.google.gwt.event.shared.HandlerManager;
import com.google.gwt.user.client.rpc.AsyncCallback;


public class ReportExporterPopup {
	
	
	private ReportExporterServiceAsync rpc = GWT.create(ReportExporterService.class);
	private HandlerManager handler;

	
	public ReportExporterPopup(final HandlerManager handler) {
		this.handler = handler;
	}
	
	public void export(final Model model, final TypeExporter type) {
		
		final Window window = new Window();  
	    window.setSize(250, 120);  
	    window.setPlain(true);
	    
	    window.setHeading("Export to " + type.toString());  
	    
	    final CheckBox instructionsCheckBox = new CheckBox();
	    instructionsCheckBox.setBoxLabel("Instructions");
	    
	    final CheckBox commentsCheckBox = new CheckBox();
	    commentsCheckBox.setBoxLabel("Comments");
	    
	    window.add(instructionsCheckBox);
	    window.add(commentsCheckBox);
	    
	    window.addButton(new Button("Cancel", new SelectionListener<ButtonEvent>()  {

			@Override
			public void componentSelected(ButtonEvent ce) {
				window.hide();
			}
	    	
	    }));
	    
	    window.addButton(new Button("Ok", new SelectionListener<ButtonEvent>()  {
			@Override
			public void componentSelected(ButtonEvent ce) {
				
				window.hide();
				final MessageBox box = MessageBox.wait("Progress",  
			            "Processing your data, please wait...", "Processing...");  
				rpc.convert(model, instructionsCheckBox.getValue(),
						commentsCheckBox.getValue(), type, new AsyncCallback<String>() {

							@Override
							public void onFailure(Throwable caught) {
								
								box.close();							
								MessageBox.alert("Alert", caught.getMessage(), null);  
								
							}

							@Override
							public void onSuccess(final String result) {
								
								box.close();
								showSaveDialog(result, model.getTemplateName(), type);
								    								
							}
				});
			}
	    	
	    }));

	    window.show();
	 
	}
	
	public class SaveDialogCallBack implements AsyncCallback<String> {

		private final Window saveDialog;
		private final OperationResult operation;
		
		public SaveDialogCallBack(final Window saveDialog, final OperationResult operation) {
			this.saveDialog = saveDialog;
			this.operation = operation;
		}
		
		@Override
		public void onFailure(
				Throwable caught) {
			
			handler.fireEvent(new ReportExporterEvent(OperationResult.FAILURE, null));
			
			if (caught instanceof SaveReportFileExistException) {
				MessageBox.alert("Alert", "A file with the same name exists in your workspace root folder," +
						" please use \"Save As\" or remove it first.", null);  
			} else {
				MessageBox.alert("Alert", "Error to save file", null);
			}
				
		}

		@Override
		public void onSuccess(String result) {
			
			saveDialog.hide();
			
			handler.fireEvent(new ReportExporterEvent(operation, result));
			MessageBox.info("Info", "File has been saved successfully", null);
		}
		
	}
	/**
	 * 
	 * @param filePath
	 * @param itemName
	 * @param type
	 */
	private void showSaveDialog(final String filePath, final String itemName, final TypeExporter type) {		
		handler.fireEvent(new ExportingCompletedEvent(filePath, itemName, type));
		
//		final Window saveDialog = new Window();  
//		saveDialog.setHeading("Save dialog"); 
//		saveDialog.addText("Click Save to save in your root folder," +
//				" Save As to specify a location.");
//		
//		saveDialog.setSize(300, 200);
//		
//		saveDialog.addButton(new Button("Save & Open", new SelectionListener<ButtonEvent>() {  
//			public void componentSelected(ButtonEvent ce) {  
//				
//				//saveDialog.hide();
//				rpc.save(filePath, null, itemName, type,new SaveDialogCallBack(saveDialog,OperationResult.SAVED_OPEN));
//			}  
//		})); 
//		
//		saveDialog.addButton(new Button("Save", new SelectionListener<ButtonEvent>() {  
//			public void componentSelected(ButtonEvent ce) {  
//				
//				//saveDialog.hide();
//				rpc.save(filePath, null, itemName, type,new SaveDialogCallBack(saveDialog,OperationResult.SAVED));
//			}  
//		})); 
//		
//		saveDialog.addButton(new Button("Save as", new SelectionListener<ButtonEvent>() {  
//			public void componentSelected(ButtonEvent ce) { 
//				
//				//saveDialog.hide();
//				GWT.runAsync(WorkspaceLightTreeSavePopup.class, new RunAsyncCallback() {
//					public void onSuccess() {
//						WorkspaceLightTreeSavePopup wpTree = new WorkspaceLightTreeSavePopup("Save file, choose folder please:", true);
//						wpTree.setSelectableTypes(ItemType.FOLDER, ItemType.ROOT);
//						wpTree.setShowEmptyFolders(true);
//						wpTree.addPopupHandler(new PopupHandler(){
//
//							@Override
//							public void onPopup(
//									PopupEvent event) {
//									
//								rpc.save(filePath, event.getSelectedItem().getId(),
//										event.getName(),type, new SaveDialogCallBack(saveDialog, OperationResult.SAVED));								
//							}});	
//						
//						wpTree.show();
//						wpTree.center();
//					}
//
//					public void onFailure(Throwable reason) {
//						      
//						MessageBox.alert("Alert", reason.getMessage(), null);  
//					}
//				});
//				
//			}  
//		})); 
//
//		saveDialog.show();
	}
	
	  
	
}
